using System;
using MicrosoftResearch.Infer;
using MicrosoftResearch.Infer.Distributions;
using MicrosoftResearch.Infer.Maths;
using MicrosoftResearch.Infer.Collections;
using MicrosoftResearch.Infer.Factors;

namespace MicrosoftResearch.Infer.Models.User
{
	/// <summary>
	/// Generated algorithm for performing inference
	/// </summary>
	/// <remarks>
	/// The easiest way to use this class is to wrap an instance in a CompiledAlgorithm object and use
	/// the methods on CompiledAlgorithm to set parameters and execute inference.
	/// 
	/// If you instead wish to use this class directly, you must perform the following steps:
	/// 1) Create an instance of the class
	/// 2) Set the value of any externally-set fields e.g. data, priors
	/// 3) Call the Execute(numberOfIterations) method
	/// 4) Use the XXXMarginal() methods to retrieve posterior marginals for different variables.
	/// 
	/// Generated by Infer.NET 2.5 at 02:52 on sexta-feira, 6 de Setembro de 2013.
	/// </remarks>
	public partial class Model5_EP : IGeneratedAlgorithm
	{
		#region Fields
		/// <summary>Field backing the NumberOfIterationsDone property</summary>
		private int numberOfIterationsDone;
		/// <summary>Field backing the prior property</summary>
		private SparseGP Prior;
		/// <summary>Field backing the routineFeaturesTest property</summary>
		private Vector[] RoutineFeaturesTest;
		/// <summary>The number of iterations last computed by Changed_prior_routineFeaturesTest. Set this to zero to force re-execution of Changed_prior_routineFeaturesTest</summary>
		public int Changed_prior_routineFeaturesTest_iterationsDone;
		public PointMass<SparseGP> prior_marginal;
		public PointMass<Vector[]> routineFeaturesTest_marginal;
		/// <summary>Message to marginal of 'f'</summary>
		public SparseGP f_marginal_F;
		/// <summary>Message to marginal of 'observedCountsTest'</summary>
		public DistributionStructArray<Gaussian,double> observedCountsTest_marginal_F;
		#endregion

		#region Properties
		/// <summary>The number of iterations done from the initial state</summary>
		public int NumberOfIterationsDone
		{			get {
				return this.numberOfIterationsDone;
			}
		}

		/// <summary>The externally-specified value of 'prior'</summary>
		public SparseGP prior
		{			get {
				return this.Prior;
			}
			set {
				this.Prior = value;
				this.numberOfIterationsDone = 0;
				this.Changed_prior_routineFeaturesTest_iterationsDone = 0;
			}
		}

		/// <summary>The externally-specified value of 'routineFeaturesTest'</summary>
		public Vector[] routineFeaturesTest
		{			get {
				return this.RoutineFeaturesTest;
			}
			set {
				if ((value!=null)&&(value.Length!=1729)) {
					throw new ArgumentException(((("Provided array of length "+value.Length)+" when length ")+1729)+" was expected for variable \'routineFeaturesTest\'");
				}
				this.RoutineFeaturesTest = value;
				this.numberOfIterationsDone = 0;
				this.Changed_prior_routineFeaturesTest_iterationsDone = 0;
			}
		}

		#endregion

		#region Methods
		/// <summary>Get the observed value of the specified variable.</summary>
		/// <param name="variableName">Variable name</param>
		public object GetObservedValue(string variableName)
		{
			if (variableName=="prior") {
				return this.prior;
			}
			if (variableName=="routineFeaturesTest") {
				return this.routineFeaturesTest;
			}
			throw new ArgumentException("Not an observed variable name: "+variableName);
		}

		/// <summary>Set the observed value of the specified variable.</summary>
		/// <param name="variableName">Variable name</param>
		/// <param name="value">Observed value</param>
		public void SetObservedValue(string variableName, object value)
		{
			if (variableName=="prior") {
				this.prior = (SparseGP)value;
				return ;
			}
			if (variableName=="routineFeaturesTest") {
				this.routineFeaturesTest = (Vector[])value;
				return ;
			}
			throw new ArgumentException("Not an observed variable name: "+variableName);
		}

		/// <summary>The marginal distribution of the specified variable.</summary>
		/// <param name="variableName">Variable name</param>
		public object Marginal(string variableName)
		{
			if (variableName=="prior") {
				return this.PriorMarginal();
			}
			if (variableName=="routineFeaturesTest") {
				return this.RoutineFeaturesTestMarginal();
			}
			if (variableName=="f") {
				return this.FMarginal();
			}
			if (variableName=="observedCountsTest") {
				return this.ObservedCountsTestMarginal();
			}
			throw new ArgumentException("This class was not built to infer "+variableName);
		}

		public T Marginal<T>(string variableName)
		{
			return Distribution.ChangeType<T>(this.Marginal(variableName));
		}

		/// <summary>The query-specific marginal distribution of the specified variable.</summary>
		/// <param name="variableName">Variable name</param>
		/// <param name="query">QueryType name. For example, GibbsSampling answers 'Marginal', 'Samples', and 'Conditionals' queries</param>
		public object Marginal(string variableName, string query)
		{
			if (query=="Marginal") {
				return this.Marginal(variableName);
			}
			throw new ArgumentException(((("This class was not built to infer \'"+variableName)+"\' with query \'")+query)+"\'");
		}

		public T Marginal<T>(string variableName, string query)
		{
			return Distribution.ChangeType<T>(this.Marginal(variableName, query));
		}

		/// <summary>Update all marginals, by iterating message passing the given number of times</summary>
		/// <param name="numberOfIterations">The number of times to iterate each loop</param>
		/// <param name="initialise">If true, messages that initialise loops are reset when observed values change</param>
		private void Execute(int numberOfIterations, bool initialise)
		{
			this.Changed_prior_routineFeaturesTest();
			this.numberOfIterationsDone = numberOfIterations;
		}

		public void Execute(int numberOfIterations)
		{
			this.Execute(numberOfIterations, true);
		}

		public void Update(int additionalIterations)
		{
			this.Execute(this.numberOfIterationsDone+additionalIterations, false);
		}

		private void OnProgressChanged(ProgressChangedEventArgs e)
		{
			// Make a temporary copy of the event to avoid a race condition
			// if the last subscriber unsubscribes immediately after the null check and before the event is raised.
			EventHandler<ProgressChangedEventArgs> handler = this.ProgressChanged;
			if (handler!=null) {
				handler(this, e);
			}
		}

		/// <summary>Reset all messages to their initial values.  Sets NumberOfIterationsDone to 0.</summary>
		public void Reset()
		{
			this.Execute(0);
		}

		/// <summary>Computations that depend on the observed value of prior and routineFeaturesTest</summary>
		public void Changed_prior_routineFeaturesTest()
		{
			if (this.Changed_prior_routineFeaturesTest_iterationsDone==1) {
				return ;
			}
			this.prior_marginal = new PointMass<SparseGP>(this.Prior);
			this.routineFeaturesTest_marginal = new PointMass<Vector[]>(this.RoutineFeaturesTest);
			this.f_marginal_F = ArrayHelper.MakeUniform<SparseGP>(this.Prior);
			// Buffer for ReplicateOp_Divide.Marginal<SparseGP>
			SparseGP f_rep_B_toDef = default(SparseGP);
			// Message to 'f_rep' from Replicate factor
			f_rep_B_toDef = ReplicateOp_Divide.ToDefInit<SparseGP>(this.Prior);
			// Message to 'f_marginal' from Variable factor
			this.f_marginal_F = VariableOp.MarginalAverageConditional<SparseGP>(f_rep_B_toDef, this.Prior, this.f_marginal_F);
			DistributionStructArray<Gaussian,double> observedCountsTest_F = default(DistributionStructArray<Gaussian,double>);
			// Create array for 'observedCountsTest' Forwards messages.
			observedCountsTest_F = new DistributionStructArray<Gaussian,double>(1729);
			for(int Ntest = 0; Ntest<1729; Ntest++) {
				observedCountsTest_F[Ntest] = Gaussian.Uniform();
			}
			// Create array for replicates of 'vdouble118_F'
			DistributionStructArray<Gaussian,double> vdouble118_F = new DistributionStructArray<Gaussian,double>(1729);
			for(int Ntest = 0; Ntest<1729; Ntest++) {
				vdouble118_F[Ntest] = Gaussian.Uniform();
			}
			DistributionRefArray<SparseGP,IFunction> f_rep_F = default(DistributionRefArray<SparseGP,IFunction>);
			DistributionRefArray<SparseGP,IFunction> f_rep_B = default(DistributionRefArray<SparseGP,IFunction>);
			// Create array for 'f_rep' Forwards messages.
			f_rep_F = new DistributionRefArray<SparseGP,IFunction>(1729);
			// Create array for 'f_rep' Backwards messages.
			f_rep_B = new DistributionRefArray<SparseGP,IFunction>(1729);
			for(int Ntest = 0; Ntest<1729; Ntest++) {
				f_rep_B[Ntest] = ArrayHelper.MakeUniform<SparseGP>(this.Prior);
				f_rep_F[Ntest] = ArrayHelper.MakeUniform<SparseGP>(this.Prior);
			}
			// Buffer for ReplicateOp_Divide.UsesAverageConditional<SparseGP>
			SparseGP f_rep_F_marginal = default(SparseGP);
			// Message to 'f_rep' from Replicate factor
			f_rep_F_marginal = ReplicateOp_Divide.MarginalInit<SparseGP>(this.Prior);
			// Message to 'f_rep' from Replicate factor
			f_rep_F_marginal = ReplicateOp_Divide.Marginal<SparseGP>(f_rep_B_toDef, this.Prior, f_rep_F_marginal);
			for(int Ntest = 0; Ntest<1729; Ntest++) {
				// Message to 'f_rep' from Replicate factor
				f_rep_F[Ntest] = ReplicateOp_Divide.UsesAverageConditional<SparseGP>(f_rep_B[Ntest], f_rep_F_marginal, Ntest, f_rep_F[Ntest]);
				// Message to 'vdouble118' from FunctionEvaluate factor
				vdouble118_F[Ntest] = SparseGPOp.YAverageConditional(f_rep_F[Ntest], this.RoutineFeaturesTest[Ntest]);
			}
			// Create array for 'observedCountsTest_marginal' Forwards messages.
			this.observedCountsTest_marginal_F = new DistributionStructArray<Gaussian,double>(1729);
			for(int Ntest = 0; Ntest<1729; Ntest++) {
				this.observedCountsTest_marginal_F[Ntest] = Gaussian.Uniform();
			}
			// Message from use of 'observedCountsTest'
			DistributionStructArray<Gaussian,double> observedCountsTest_use_B = default(DistributionStructArray<Gaussian,double>);
			// Create array for 'observedCountsTest_use' Backwards messages.
			observedCountsTest_use_B = new DistributionStructArray<Gaussian,double>(1729);
			for(int Ntest = 0; Ntest<1729; Ntest++) {
				observedCountsTest_use_B[Ntest] = Gaussian.Uniform();
				// Message to 'observedCountsTest' from GaussianFromMeanAndVariance factor
				observedCountsTest_F[Ntest] = GaussianFromMeanAndVarianceOp.SampleAverageConditional(vdouble118_F[Ntest], 0.1);
				// Message to 'observedCountsTest_marginal' from Variable factor
				this.observedCountsTest_marginal_F[Ntest] = VariableOp.MarginalAverageConditional<Gaussian>(observedCountsTest_use_B[Ntest], observedCountsTest_F[Ntest], this.observedCountsTest_marginal_F[Ntest]);
			}
			this.Changed_prior_routineFeaturesTest_iterationsDone = 1;
		}

		/// <summary>
		/// Returns the marginal distribution for 'prior' given by the current state of the
		/// message passing algorithm.
		/// </summary>
		/// <returns>The marginal distribution</returns>
		public PointMass<SparseGP> PriorMarginal()
		{
			return this.prior_marginal;
		}

		/// <summary>
		/// Returns the marginal distribution for 'routineFeaturesTest' given by the current state of the
		/// message passing algorithm.
		/// </summary>
		/// <returns>The marginal distribution</returns>
		public PointMass<Vector[]> RoutineFeaturesTestMarginal()
		{
			return this.routineFeaturesTest_marginal;
		}

		/// <summary>
		/// Returns the marginal distribution for 'f' given by the current state of the
		/// message passing algorithm.
		/// </summary>
		/// <returns>The marginal distribution</returns>
		public SparseGP FMarginal()
		{
			return this.f_marginal_F;
		}

		/// <summary>
		/// Returns the marginal distribution for 'observedCountsTest' given by the current state of the
		/// message passing algorithm.
		/// </summary>
		/// <returns>The marginal distribution</returns>
		public DistributionStructArray<Gaussian,double> ObservedCountsTestMarginal()
		{
			return this.observedCountsTest_marginal_F;
		}

		#endregion

		#region Events
		/// <summary>Event that is fired when the progress of inference changes, typically at the end of one iteration of the inference algorithm.</summary>
		public event EventHandler<ProgressChangedEventArgs> ProgressChanged;
		#endregion

	}

}
