import sys
import numpy as np
import sklearn.cluster

input_file = sys.argv[1]
output_file = input_file.replace(".tsv","_fake.tsv")
answers_file = input_file.replace(".tsv","_fake_answers.tsv")

f = open(input_file)
f.readline() # skip header
cases = []
venues_dict = {}
next_venue_id = 0
categories_dict = {}
next_category_id = 0
for line in f:
	split = line.replace("\n","").replace("\r","").split("\t")
	stop_id,date,dow,holiday,start_time,end_time,area,rmse,mae,ae,overall_mean,expected_mean,observed_mean = split[:13]
	events = split[13:]
	print stop_id,date,start_time,end_time
	cases.append((stop_id,date,dow,holiday,start_time,end_time,area,rmse,mae,ae,overall_mean,expected_mean,observed_mean,events))
	for e in events:
		_id,title,venue,distance,all_day,repeated_count,categories,topics,ghits1,ghits2,fb1,fb2,fb3 = e.split(",")
		num_topics = len(topics.split("|"))
		print "\t", title
		categories = categories.split("|")
		for cat in categories:
			if cat not in categories_dict:
				categories_dict[cat] = next_category_id
				next_category_id += 1
		if venue not in venues_dict:
			venues_dict[venue] = next_venue_id
			next_venue_id += 1
f.close()
print

num_features = len(venues_dict) + len(categories_dict) + num_topics + 7
events_ids = []
events_vectors = []
for case in cases:
	stop_id,date,dow,holiday,start_time,end_time,area,rmse,mae,ae,overall_mean,expected_mean,observed_mean,events = case
	for e in events:
		_id,title,venue,distance,all_day,repeated_count,categories,topics,ghits1,ghits2,fb1,fb2,fb3 = e.split(",")
		print "\t", title
		vector = np.zeros(num_features)
		
		offset = 0
		vector[offset + venues_dict[venue]] = 1
		offset += len(venues_dict)
		
		for cat in categories.split("|"):
			vector[offset + categories_dict[cat]] = 1
		offset += len(categories_dict)
		
		if all_day == "True":
			vector[offset] = 1
		offset += 1
		
		if repeated_count != "None":
			vector[offset] = int(repeated_count)
		offset += 1
		
		i = 0
		for t in topics.split("|"):
			vector[offset + i] = float(t)
			i += 1
		offset += len(topics.split("|"))
		
		vector[offset] = int(ghits1)
		vector[offset+1] = int(ghits2)
		vector[offset+2] = int(fb1)
		vector[offset+3] = int(fb2)
		vector[offset+4] = int(fb3)
		events_ids.append(_id)
		events_vectors.append(vector)

x = np.array(events_vectors)
std = x.std(axis=0)
std[np.where(std==0)] = 1
x = (x - x.mean(axis=0)) / std
kmeans = sklearn.cluster.KMeans(n_clusters=5)
kmeans.fit(x)
cluster_assignments = kmeans.predict(x)
scores = kmeans.transform(x).min(axis=1)
print cluster_assignments
print scores
print 1/scores

cluster_impacts = {0:2000,1:1000,2:400,3:200,4:10}
events_impacts = {}
for i in xrange(len(events_ids)):
	_id = events_ids[i]
	cluster = cluster_assignments[i]
	cluster_impact = cluster_impacts[cluster]
	#score = (1.0/scores[i]) + 1.0
	score = 10.0/scores[i]
	impact = score * cluster_impact
	#impact = cluster_impact
	print _id, cluster, cluster_impact, score, impact
	events_impacts[_id] = impact

fw = open(output_file, "w")
fw_answers = open(answers_file, "w")
for case in cases:
	stop_id,date,dow,holiday,start_time,end_time,area,rmse,mae,ae,overall_mean,expected_mean,observed_mean,events = case
	#fake_ae = float(dow) * np.random.normal(10,2)
	fake_ae = 10 + np.random.normal(0,10)
	#if np.random.rand() < 0.05:
	#	fake_ae += np.random.rand() * 500
	fw_answers.write("%s\t%s\t%s\t%s\t%s" % (stop_id,date,start_time,end_time,str(fake_ae)))
	for e in events:
		_id,title,venue,distance,all_day,repeated_count,categories,topics,ghits1,ghits2,fb1,fb2,fb3 = e.split(",")
		impact = events_impacts[_id]
		fake_ae += impact
		fw_answers.write("\t%s,%s" % (title,impact))
	fw_answers.write("\tTOTAL=%s\n" % (str(fake_ae),))
	events_str = ""
	for e in events:
		events_str += "\t" + e
	fw.write("%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s%s\n" % (stop_id,date,dow,holiday,start_time,end_time,area,rmse,mae,str(fake_ae),1,1,1,events_str))

fw.close()
fw_answers.close()
